﻿Imports System
Imports System.Text
Imports System.Runtime.InteropServices
Imports System.Collections.Generic
Imports System.ComponentModel

' This code module comes from this Google Forum posting:
'http://groups.google.com/group/microsoft.public.dotnet.framework/msg/99a5dd5c80c084b5?q=%22structured+storage%22+c%23&start=10&hl=en&lr&ie=UTF-8&oe=UTF-8&rnum=17
'The code is originally C# (and that was converted from VB.NET!) so I used an online convertor to
'convert it back to VB.NET for use here.
'The original inspiration for the code the Google Groups user was this example:
'http://www.mvps.org/emorcillo/en/code/grl/storage.shtml
Namespace StructuredStorageWrapper
    Public Enum SummaryPropId As UInteger
        Title = &H2
        Subject = &H3
        Author = &H4
        Keywords = &H5
        Comments = &H6
        Template = &H7
        LastSavedBy = &H8
        RevisionNumber = &H9
        TotalEditingTime = &HA
        LastPrinted = &HB
        CreateDateTime = &HC
        LastSaveDateTime = &HD
        NumPages = &HE
        NumWords = &HF
        NumChars = &H10
        Thumbnail = &H11
        AppName = &H12
        Security = &H13
    End Enum

    Public Enum STGC As Int32
        [DEFAULT] = 0
        OVERWRITE = 1
        ONLYIFCURRENT = 2
        DANGEROUSLYCOMMITMERELYTODISKCACHE = 4
        CONSOLIDATE = 8
    End Enum

    Public Enum PROPSETFLAG As Int32
        [DEFAULT] = 0
        NONSIMPLE = 1
        ANSI = 2
        UNBUFFERED = 4
        CASE_SENSITIVE = 8
    End Enum


    Public Enum STGM As Int32
        READ = &H0
        WRITE = &H1
        READWRITE = &H2
        SHARE_DENY_NONE = &H40
        SHARE_DENY_READ = &H30
        SHARE_DENY_WRITE = &H20
        SHARE_EXCLUSIVE = &H10
        PRIORITY = &H40000
        CREATE = &H1000
        CONVERT = &H20000
        FAILIFTHERE = &H0
        DIRECT = &H0
        TRANSACTED = &H10000
        NOSCRATCH = &H100000
        NOSNAPSHOT = &H200000
        SIMPLE = &H8000000
        DIRECT_SWMR = &H400000
        DELETEONRELEASE = &H4000000
    End Enum


    Public Enum STGFMT As Int32
        STORAGE = 0
        FILE = 3
        ANY = 4
        DOCFILE = 5
    End Enum

    <StructLayout(LayoutKind.Explicit, Size:=8, CharSet:=CharSet.Unicode)> _
    Public Structure PropSpec
        <FieldOffset(0)> _
        Public ulKind As Integer
        <FieldOffset(4)> _
        Public Name_Or_ID As IntPtr
    End Structure

    <StructLayout(LayoutKind.Explicit, Size:=16)> _
    Public Structure PropVariant
        <FieldOffset(0)> _
        Public variantType As Short
        <FieldOffset(8)> _
        Public pointerValue As IntPtr
        <FieldOffset(8)> _
        Public byteValue As Byte
        <FieldOffset(8)> _
        Public longValue As Long

        Public Sub FromObject(ByVal obj As Object)
            If obj.GetType Is GetType(String) Then
                Me.variantType = CType(System.Runtime.InteropServices.VarEnum.VT_LPWSTR, Short)
                Me.pointerValue = Marshal.StringToHGlobalUni(CType(obj, String))
            End If
        End Sub
    End Structure

    <ComVisible(True), ComImport(), Guid("0000013A-0000-0000-C000-000000000046"), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IPropertySetStorage

        Function Create(<[In](), MarshalAs(UnmanagedType.Struct)> _
           ByRef rfmtid As System.Guid, <[In]()> _
           ByVal pclsid As IntPtr, <[In]()> _
           ByVal grfFlags As Integer, <[In]()> _
           ByVal grfMode As Integer, ByRef propertyStorage As IPropertyStorage) As System.UInt32

        Function Open(<[In](), MarshalAs(UnmanagedType.Struct)> _
            ByRef rfmtid As System.Guid, <[In]()> _
            ByVal grfMode As Integer, <Out()> _
            ByVal propertyStorage As IPropertyStorage) As Integer
    End Interface


    <ComVisible(True), ComImport(), Guid("00000138-0000-0000-C000-000000000046"), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IPropertyStorage

        Function ReadMultiple(ByVal numProperties As System.UInt32, ByVal propertySpecifications As PropSpec(), ByVal propertyValues As PropVariant()) As Integer

        Function WriteMultiple(ByVal numProperties As System.UInt32, <MarshalAs(UnmanagedType.Struct)> _
            ByRef propertySpecification As PropSpec, ByRef propertyValues As PropVariant, ByVal propIDNameFirst As Integer) As Integer

        Function Commit(ByVal commitFlags As Integer) As System.UInt32
    End Interface

    Public Enum HResults As Int32
        S_OK = 0
        STG_E_FILEALREADYEXISTS = &H80030050
    End Enum

    Public Class ole32

        <StructLayout(LayoutKind.Explicit, Size:=12, CharSet:=CharSet.Unicode)> _
        Public Structure STGOptions
            <FieldOffset(0)> _
            Private usVersion As System.UInt16
            <FieldOffset(2)> _
            Private reserved As System.UInt16
            <FieldOffset(4)> _
            Private uiSectorSize As System.UInt32
            <FieldOffset(8), MarshalAs(UnmanagedType.LPWStr)> _
            Private pwcsTemplateFile As String
        End Structure

        <DllImport("ole32.dll", CharSet:=CharSet.Unicode)> _
        Public Shared Function StgCreateStorageEx(<MarshalAs(UnmanagedType.LPWStr)> ByVal name As String, ByVal accessMode As Integer, ByVal storageFileFormat As Integer, ByVal fileBuffering As Integer, ByVal options As IntPtr, ByVal reserved As IntPtr, ByRef riid As System.Guid, <MarshalAs(UnmanagedType.Interface)> ByRef propertySetStorage As IPropertySetStorage) As System.UInt32
        End Function

        <DllImport("ole32.dll", CharSet:=CharSet.Unicode)> _
        Public Shared Function StgOpenStorageEx(<MarshalAs(UnmanagedType.LPWStr)> ByVal name As String, ByVal accessMode As Integer, ByVal storageFileFormat As Integer, ByVal fileBuffering As Integer, ByVal options As IntPtr, ByVal reserved As IntPtr, ByRef riid As System.Guid, <MarshalAs(UnmanagedType.Interface)> ByRef propertySetStorage As IPropertySetStorage) As System.UInt32
        End Function
    End Class
End Namespace
Public Module clsStructStorage
    Sub SaveTags(ByVal theTags As String, ByVal theFile As String)
        Try
            Dim propSetStorage As StructuredStorageWrapper.IPropertySetStorage = Nothing
            Dim IID_PropertySetStorage As Guid = New Guid("0000013A-0000-0000-C000-000000000046")
            Dim hresult As System.UInt32 = StructuredStorageWrapper.ole32.StgOpenStorageEx(theFile, CType((StructuredStorageWrapper.STGM.SHARE_EXCLUSIVE Or StructuredStorageWrapper.STGM.READWRITE), Integer), CType(StructuredStorageWrapper.STGFMT.FILE, Integer), 0, CType(0, IntPtr), CType(0, IntPtr), IID_PropertySetStorage, propSetStorage)
            Dim fmtid_SummaryProperties As Guid = New Guid("F29F85E0-4FF9-1068-AB91-08002B27B3D9")
            Dim propStorage As StructuredStorageWrapper.IPropertyStorage = Nothing

            hresult = propSetStorage.Create(fmtid_SummaryProperties, CType(0, IntPtr), CType(StructuredStorageWrapper.PROPSETFLAG.DEFAULT, Integer), CType((StructuredStorageWrapper.STGM.CREATE Or StructuredStorageWrapper.STGM.READWRITE Or StructuredStorageWrapper.STGM.SHARE_EXCLUSIVE), Integer), propStorage)

            Dim propertySpecification As StructuredStorageWrapper.PropSpec = New StructuredStorageWrapper.PropSpec

            propertySpecification.ulKind = 1
            propertySpecification.Name_Or_ID = New IntPtr(CType(StructuredStorageWrapper.SummaryPropId.Keywords, Integer))

            Dim propertyValue As StructuredStorageWrapper.PropVariant = New StructuredStorageWrapper.PropVariant

            propertyValue.FromObject(theTags)
            propStorage.WriteMultiple(1, propertySpecification, propertyValue, 2)

            hresult = propStorage.Commit(CType(StructuredStorageWrapper.STGC.DEFAULT, Integer))
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Function ReadTags(ByVal theFile As String) As String
        Try
            Dim propSetStorage As StructuredStorageWrapper.IPropertySetStorage = Nothing
            Dim IID_PropertySetStorage As Guid = New Guid("0000013A-0000-0000-C000-000000000046")
            Dim hresult As System.UInt32 = StructuredStorageWrapper.ole32.StgOpenStorageEx(theFile, CType((StructuredStorageWrapper.STGM.SHARE_EXCLUSIVE Or StructuredStorageWrapper.STGM.READWRITE), Integer), CType(StructuredStorageWrapper.STGFMT.FILE, Integer), 0, CType(0, IntPtr), CType(0, IntPtr), IID_PropertySetStorage, propSetStorage)
            Dim fmtid_SummaryProperties As Guid = New Guid("F29F85E0-4FF9-1068-AB91-08002B27B3D9")
            Dim propStorage As StructuredStorageWrapper.IPropertyStorage = Nothing

            hresult = propSetStorage.Create(fmtid_SummaryProperties, CType(0, IntPtr), CType(StructuredStorageWrapper.PROPSETFLAG.DEFAULT, Integer), CType((StructuredStorageWrapper.STGM.CREATE Or StructuredStorageWrapper.STGM.READWRITE Or StructuredStorageWrapper.STGM.SHARE_EXCLUSIVE), Integer), propStorage)

            'Dim propertySpecs As New StructuredStorageWrapper.PropSpec

            'propertySpecs.ulKind = 1
            'propertySpecs.Name_Or_ID = New IntPtr(CType(StructuredStorageWrapper.SummaryPropId.Keywords, Integer))

            'Dim propertyValue As StructuredStorageWrapper.PropVariant = New StructuredStorageWrapper.PropVariant

            Dim pscSpecs As StructuredStorageWrapper.PropSpec() = New StructuredStorageWrapper.PropSpec(0) {}
            Dim pvValues As StructuredStorageWrapper.PropVariant() = New StructuredStorageWrapper.PropVariant(0) {}

            ' Specify the property to be retrieved 
            pscSpecs(0).ulKind = 1
            pscSpecs(0).Name_Or_ID = New IntPtr(CType(StructuredStorageWrapper.SummaryPropId.Keywords, Integer))
            propStorage.ReadMultiple(1, pscSpecs, pvValues)

            Return Marshal.PtrToStringUni(pvValues(0).pointerValue)
        Catch ex As Exception
            globalErrorHandler(ex)
            Return String.Empty
        End Try
    End Function

    Public Function GetProperty(ByVal summaryPropertID As StructuredStorageWrapper.SummaryPropId) As String
        'This code block was taken from this forum thread:
        'http://www.codeproject.com/Questions/59152/How-to-use-ipropertystorage
        Dim pssSetStorage As StructuredStorageWrapper.IPropertySetStorage = Nothing
        Dim guidPropertySetStorage As New Guid("0000013A-0000-0000-C000-000000000046")
        Dim guidSummaryProperties As New Guid("F29F85E0-4FF9-1068-AB91-08002B27B3D9")
        Dim psStorage As StructuredStorageWrapper.IPropertyStorage = Nothing
        Dim pscSpecs As StructuredStorageWrapper.PropSpec() = New StructuredStorageWrapper.PropSpec(0) {}
        Dim pvValues As StructuredStorageWrapper.PropVariant() = New StructuredStorageWrapper.PropVariant(0) {}

        Try
            'Exit if file does not exist
            'If Not fiSource.Exists Then
            'Return Nothing
            'End If
            ' Open the file and its property set stream 
            'Dim hresult As UInteger = StructuredStorageWrapper.ole32.StgOpenStorageEx(fiSource.FullName, CInt(StructuredStorageWrapper.STGM.DIRECT_SWMR Or StructuredStorageWrapper.STGM.READ Or StructuredStorageWrapper.STGM.SHARE_DENY_NONE), CInt(StructuredStorageWrapper.STGFMT.FILE), 0, System.IntPtr.Zero, System.IntPtr.Zero, _
            ' guidPropertySetStorage, pssSetStorage)
            ' Open the Summary Information property set 
            Dim hresult2 As Integer = pssSetStorage.Open(guidSummaryProperties, CInt(StructuredStorageWrapper.STGM.DIRECT Or StructuredStorageWrapper.STGM.SHARE_EXCLUSIVE), psStorage)
            ' Specify the property to be retrieved 
            pscSpecs(0).ulKind = 1
            pscSpecs(0).Name_Or_ID = New IntPtr(CInt(summaryPropertID))
            ' Retrieve the value 
            hresult2 = psStorage.ReadMultiple(1, pscSpecs, pvValues)
            ' Release the Com objects 
            Marshal.ReleaseComObject(psStorage)
            Marshal.ReleaseComObject(pssSetStorage)
            psStorage = Nothing
            pssSetStorage = Nothing
        Catch ex As Exception
            globalErrorHandler(ex)
        Finally
            pssSetStorage = Nothing
            psStorage = Nothing
            pscSpecs = Nothing
            pvValues = Nothing
        End Try
        Return Marshal.PtrToStringUni(pvValues(0).pointerValue)
    End Function

End Module
