﻿Imports System.Windows.Forms
Imports System.Drawing
Imports System.Drawing.Text
Imports System.IO
Imports Outlook = Microsoft.Office.Interop.Outlook
Imports System.Drawing.Imaging
Imports DSOFile

Public Class frmPhotoDlg
    Dim imgCol As New Collection()
    Dim photoPathCol As New Collection()
    Dim lastLat As New Double
    Dim lastLong As New Double

    Private Sub cmdSelectFolder_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdSelectFolder.Click
        Try
            ' First create a FolderBrowserDialog object
            Dim FolderBrowserDialog1 As New FolderBrowserDialog
            Dim curFileName As String

            imgCol.Clear()
            photoPathCol.Clear()

            ' Then use the following code to create the Dialog window
            ' Change the .SelectedPath property to the default location
            With FolderBrowserDialog1
                ' Desktop is the root folder in the dialog.
                .RootFolder = Environment.SpecialFolder.Desktop
                ' Select the C:\Windows directory on entry.
                If Len(Me.txtFolderPath.Text) > 0 Then
                    .SelectedPath = Me.txtFolderPath.Text
                Else
                    .SelectedPath = "s:\"
                End If
                ' Prompt the user with a custom message.
                .Description = "Select the photo folder"
                If .ShowDialog = DialogResult.OK Then
                    ' Display the selected folder if the user clicked on the OK button.
                    Me.txtFolderPath.Text = FolderBrowserDialog1.SelectedPath
                Else
                    'Dialog was cancelled so do nothing
                    Exit Sub
                End If
            End With

            'Populate the imageList control with any images found in the specified folder
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.WaitCursor

            imgCol = fillImlWithFilesFromDir(FolderBrowserDialog1.SelectedPath)
            photoPathCol = fillCollWithFilesFromDir(FolderBrowserDialog1.SelectedPath)
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.Default

            'If any images were found in the folder, autoload the first image into the dialog
            'Code source: http://www.dreamincode.net/forums/topic/105965-the-image-list/
            If imgCol.Count > 0 Then
                Dim theFileStream As New System.IO.FileStream(photoPathCol(1), FileMode.Open, FileAccess.Read)
                Dim myImage As Image = Image.FromStream(theFileStream)

                Me.imgBox.Image = myImage
                myImage = Nothing
                theFileStream.Dispose()

                Me.imgBox.Tag = 1
                Me.lblNumFound.Text = imgCol.Count & " Photo(s) Found"
                Me.lblPhotoCount.Visible = True
                Me.lblPhotoCount.Text = "Photo 1 of " & imgCol.Count
                Me.txtImgName.Text = IO.Path.GetFileNameWithoutExtension(photoPathCol(1)) 'imgBox.Image.GetPropertyItem(Val("0x010D"))
                curFileName = photoPathCol(1)

                Dim EW As New ExifWorks(curFileName)
                Dim GPSLat As Double = EW.GPSLatitude()
                Dim GPSLong As Double = EW.GPSLongitude()
                Dim imgTagList As String = ""

                Me.cmdActivateLatLong.Enabled = True
                Me.cmdFirstPhoto.Enabled = True
                Me.cmdLastPhoto.Enabled = True
                Me.cmdNextPhoto.Enabled = True
                Me.cmdPrevPhoto.Enabled = True
                Me.cmdRotImgCcw.Enabled = True
                Me.cmdRotImgCw.Enabled = True
                Me.cmdReuseLastCoord.Enabled = False

                If Not Double.IsNaN(GPSLat) Then
                    Me.txtLatitude.Text = GPSLat.ToString
                    lastLat = GPSLat
                Else
                    Me.txtLatitude.Text = ""
                End If
                If Not Double.IsNaN(GPSLong) Then
                    Me.txtLongitude.Text = GPSLong.ToString
                    lastLong = GPSLong
                Else
                    Me.txtLongitude.Text = ""
                End If

                Dim theDescBytes As Byte() = EW.GetProperty(ExifWorks.TagNames.ExifUserComment)

                If theDescBytes Is Nothing Then
                    Me.txtImgDesc.Text = String.Empty
                Else
                    Me.txtImgDesc.Text = System.Text.Encoding.Unicode.GetString(theDescBytes)
                End If

                imgTagList = EW.GetPropertyString(40094) 'ReadFileTags(curFileName)

                If Len(imgTagList) > 0 Then
                    updateExifTagsFromPhoto(imgTagList, Me.clstExifTags)
                    Me.clstExifTags.Tag = imgTagList
                Else
                    fillCheckListboxFromFile(Me.clstExifTags)
                    Me.clstExifTags.Tag = ""
                End If
            Else
                Me.imgBox.Image = Nothing
                Me.lblPhotoCount.Visible = False
                Me.lblNumFound.Text = "No photos found"
                Me.txtImgName.Text = String.Empty

                Me.cmdActivateLatLong.Enabled = False
                Me.cmdFirstPhoto.Enabled = False
                Me.cmdLastPhoto.Enabled = False
                Me.cmdNextPhoto.Enabled = False
                Me.cmdPrevPhoto.Enabled = False
                Me.cmdRotImgCcw.Enabled = False
                Me.cmdRotImgCw.Enabled = False
            End If

            'Release the folderBrowserDialog from memory
            FolderBrowserDialog1.Dispose()
        Catch ex As Exception
            Me.Cursor = Cursors.Arrow
            globalErrorHandler(ex)
        End Try
    End Sub
    Function fillImlWithFilesFromDir(ByRef dirPath As String) As Collection
        'Routine found at this website:
        'http://www.dreamincode.net/forums/topic/105965-the-image-list/
        Dim theColl As New Collection()
        Try
            Dim dirFiles() As String = IO.Directory.GetFiles(dirPath)

            For Each dirFile As String In dirFiles
                For Each extension As String In allowedExtensions
                    If extension = IO.Path.GetExtension(dirFile) Then
                        Dim theFileStream As New System.IO.FileStream(dirFile, FileMode.Open, FileAccess.Read)
                        Dim myImage As Image = Image.FromStream(theFileStream)

                        theColl.Add(myImage)
                        myImage = Nothing
                        theFileStream.Dispose()
                    End If
                Next
            Next

            Return theColl
        Catch ex As Exception
            globalErrorHandler(ex)
            Return theColl
        End Try
    End Function
    Private allowedExtensions() As String = { _
".jpg", _
".JPG", _
".JPEG", _
".jpeg" _
 }
    Function fillCollWithFilesFromDir(ByRef dirPath As String) As Collection
        'Routine found at this website:
        'http://www.dreamincode.net/forums/topic/105965-the-image-list/
        Dim theColl As New Collection()
        Try
            Dim dirFiles() As String = IO.Directory.GetFiles(dirPath)

            For Each dirFile As String In dirFiles
                For Each extension As String In allowedExtensions
                    If extension = IO.Path.GetExtension(dirFile) Then
                        theColl.Add(dirFile)
                    End If
                Next
            Next

            Return theColl
        Catch ex As Exception
            globalErrorHandler(ex)
            Return theColl
        End Try
    End Function

    Private Sub cmdNextPhoto_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdNextPhoto.Click
        Try
            Dim curImgNum As Integer
            Dim curFileName As String
            curImgNum = Me.imgBox.Tag

            Select Case curImgNum
                Case imgCol.Count
                    MsgBox("Already at the last photo!", MsgBoxStyle.Exclamation, "Last Photo")
                    Exit Sub
                Case Else
                    Dim theFileStream As New System.IO.FileStream(photoPathCol(curImgNum + 1), FileMode.Open, FileAccess.Read)
                    Dim myImage As Image = Image.FromStream(theFileStream)

                    Me.imgBox.Image = myImage
                    myImage = Nothing
                    theFileStream.Dispose()

                    'Me.imgBox.Image = imgCol(curImgNum + 1)
                    Me.txtImgName.Text = IO.Path.GetFileNameWithoutExtension(photoPathCol(curImgNum + 1))
                    Me.imgBox.Tag = curImgNum + 1
                    Me.lblPhotoCount.Text = "Photo " & Me.imgBox.Tag & " of " & imgCol.Count

                    curFileName = photoPathCol(curImgNum + 1)

                    Dim EW As New ExifWorks(curFileName)
                    Dim GPSLat As Double = EW.GPSLatitude()
                    Dim GPSLong As Double = EW.GPSLongitude()
                    Dim imgTagList As String = ""

                    If Not Double.IsNaN(GPSLat) Then
                        Me.txtLatitude.Text = GPSLat.ToString
                    Else
                        Me.txtLatitude.Text = ""
                    End If
                    If Not Double.IsNaN(GPSLong) Then
                        Me.txtLongitude.Text = GPSLong.ToString
                    Else
                        Me.txtLongitude.Text = ""
                    End If

                    If lastLat <> 0 Then
                        Me.cmdReuseLastCoord.Enabled = True
                    Else
                        Me.cmdReuseLastCoord.Enabled = False
                    End If

                    Dim theDescBytes As Byte() = EW.GetProperty(ExifWorks.TagNames.ExifUserComment)

                    If theDescBytes Is Nothing Then
                        Me.txtImgDesc.Text = String.Empty
                    Else
                        Me.txtImgDesc.Text = System.Text.Encoding.Unicode.GetString(theDescBytes)
                    End If

                    imgTagList = EW.GetPropertyString(40094) 'ReadFileTags(curFileName)

                    If Len(imgTagList) > 0 Then
                        updateExifTagsFromPhoto(imgTagList, Me.clstExifTags)
                        Me.clstExifTags.Tag = imgTagList
                    Else
                        fillCheckListboxFromFile(Me.clstExifTags)
                        Me.clstExifTags.Tag = ""
                    End If

                    If Me.cmdSaveChanges.Enabled Then Me.cmdSaveChanges.Enabled = False
                    EW = Nothing
            End Select
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub cmdPrevPhoto_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdPrevPhoto.Click
        Try
            Dim curImgNum As Integer
            Dim curFileName As String

            curImgNum = Me.imgBox.Tag

            Select Case curImgNum
                Case 1
                    MsgBox("Already at the first photo!", MsgBoxStyle.Exclamation, "First Photo")
                    Exit Sub
                Case Else
                    Dim theFileStream As New System.IO.FileStream(photoPathCol(curImgNum - 1), FileMode.Open, FileAccess.Read)
                    Dim myImage As Image = Image.FromStream(theFileStream)

                    Me.imgBox.Image = myImage
                    myImage = Nothing
                    theFileStream.Dispose()

                    'Me.imgBox.Image = imgCol(curImgNum - 1)
                    Me.txtImgName.Text = IO.Path.GetFileNameWithoutExtension(photoPathCol(curImgNum - 1))
                    Me.imgBox.Tag = curImgNum - 1
                    Me.lblPhotoCount.Text = "Photo " & Me.imgBox.Tag & " of " & imgCol.Count

                    curFileName = photoPathCol(curImgNum - 1)

                    Dim EW As New ExifWorks(curFileName)
                    Dim GPSLat As Double = EW.GPSLatitude()
                    Dim GPSLong As Double = EW.GPSLongitude()
                    Dim imgTagList As String = EW.GetPropertyString(37510, "")

                    If Not Double.IsNaN(GPSLat) Then
                        Me.txtLatitude.Text = GPSLat.ToString
                    Else
                        Me.txtLatitude.Text = ""
                    End If
                    If Not Double.IsNaN(GPSLong) Then
                        Me.txtLongitude.Text = GPSLong.ToString
                    Else
                        Me.txtLongitude.Text = ""
                    End If

                    If lastLat <> 0 Then
                        Me.cmdReuseLastCoord.Enabled = True
                    Else
                        Me.cmdReuseLastCoord.Enabled = False
                    End If

                    Dim theDescBytes As Byte() = EW.GetProperty(ExifWorks.TagNames.ExifUserComment)

                    If theDescBytes Is Nothing Then
                        Me.txtImgDesc.Text = String.Empty
                    Else
                        Me.txtImgDesc.Text = System.Text.Encoding.Unicode.GetString(theDescBytes)
                    End If

                    imgTagList = EW.GetPropertyString(40094) 'ReadFileTags(curFileName)

                    If Len(imgTagList) > 0 Then
                        updateExifTagsFromPhoto(imgTagList, Me.clstExifTags)
                        Me.clstExifTags.Tag = imgTagList
                    Else
                        fillCheckListboxFromFile(Me.clstExifTags)
                        Me.clstExifTags.Tag = ""
                    End If

                    If Me.cmdSaveChanges.Enabled Then Me.cmdSaveChanges.Enabled = False
                    EW = Nothing
            End Select
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub cmdRotImgCw_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdRotImgCw.Click
        Try
            Dim img As Image = Me.imgBox.Image
            img.RotateFlip(RotateFlipType.Rotate90FlipNone)
            Me.imgBox.Image = img
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub cmdRotImgCcw_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdRotImgCcw.Click
        Try
            Dim img As Image = Me.imgBox.Image
            img.RotateFlip(RotateFlipType.Rotate270FlipNone)
            Me.imgBox.Image = img
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub cmdSaveChanges_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdSaveChanges.Click
        Try
            Dim ver(4) As Byte
            Dim theLatValue As Decimal
            Dim theLongValue As Decimal

            Dim curFileName As String
            Dim newFileName As String
            Dim curImgNum As Integer
            Dim theLong(24) As Byte
            Dim theLat(24) As Byte
            Dim coord(6) As UInteger
            Dim theTagList As String

            ver(0) = 2
            ver(1) = 2
            ver(2) = 0
            ver(3) = 0


            '----------------------------------------------------------------------
            ' First make sure GPS coordinates have been specified. If not, abort
            '----------------------------------------------------------------------
            If Not Microsoft.VisualBasic.IsNumeric(Me.txtLongitude.Text) Or Not Microsoft.VisualBasic.IsNumeric(Me.txtLatitude.Text) Then
                MsgBox("You must specify numbers for both latitude and longitude in order to complete the update process." & _
                       vbNewLine & vbNewLine & "Please specify coordinates and try again.", MsgBoxStyle.Exclamation, "Oops- Missing Coordinates")
                Exit Sub
            End If

            Dim theLongRef = New Byte() {Convert.ToByte(If(Me.txtLongitude.Text < 0, "W"c, "E"c)), 0}
            Dim theLatRef = New Byte() {Convert.ToByte(If(Me.txtLatitude.Text < 0, "S"c, "N"c)), 0}

            theLongValue = Convert.ToDecimal(Me.txtLongitude.Text)
            theLatValue = Convert.ToDecimal(Me.txtLatitude.Text)

            If theLongValue = 0 Or theLatValue = 0 Then
                MsgBox("You must specify a latitude and longitude in order to complete the update process." & _
                       vbNewLine & vbNewLine & "Please specify coordinates and try again.", MsgBoxStyle.Exclamation, "Oops- Missing Coordinates")
                Exit Sub
            End If

            '----------------------------------------------------------------------
            ' Obtain a reference to the current image on the form and create the
            ' filename for the updated (edited) version)
            '----------------------------------------------------------------------
            curImgNum = Me.imgBox.Tag
            curFileName = photoPathCol(curImgNum)
            newFileName = System.IO.Path.GetDirectoryName(curFileName) & "\" & System.IO.Path.GetFileNameWithoutExtension(curFileName) & "_gps.jpg"


            '----------------------------------------------------------------------
            ' The lat/long values must be converted to a rational data type before
            ' they can be inserted into the image's EXIF structure
            '----------------------------------------------------------------------
            If theLongValue < 0 Then theLongValue = -1 * theLongValue
            If theLatValue < 0 Then theLatValue = -1 * theLatValue

            Call convertDec2CoordAndBytes(theLatValue, coord, theLat)
            Call convertDec2CoordAndBytes(theLongValue, coord, theLong)


            '----------------------------------------------------------------------
            ' Using the ExifWorks tools, update the various EXIF properties and
            ' save the edits into a new JPEG file
            '----------------------------------------------------------------------
            Dim EW As New ExifWorks(curFileName)

            EW.SetProperty(CInt(ExifWorks.TagNames.GpsVer), ver, ExifWorks.ExifDataTypes.SignedByte)
            EW.SetProperty(CInt(ExifWorks.TagNames.GpsLongitudeRef), theLongRef, ExifWorks.ExifDataTypes.AsciiString)
            EW.SetProperty(CInt(ExifWorks.TagNames.GpsLatitudeRef), theLatRef, ExifWorks.ExifDataTypes.AsciiString)

            EW.SetProperty(CInt(ExifWorks.TagNames.GpsLongitude), theLong, ExifWorks.ExifDataTypes.UnsignedRational)
            EW.SetProperty(CInt(ExifWorks.TagNames.GpsLatitude), theLat, ExifWorks.ExifDataTypes.UnsignedRational)

            EW.SetPropertyString(ExifWorks.TagNames.ExifUserComment, Me.txtImgDesc.Text)

            'Add Author information..
            Dim fullName As String
            Dim uLogin As String
            Dim olContactInfo As New oLookContactInfo

            uLogin = System.Environment.UserName
            olContactInfo = getOutlookName()

            fullName = olContactInfo.fullName

            'MsgBox("Department: " & olContactInfo.department & vbNewLine & _
            '        "Division: " & olContactInfo.division, MsgBoxStyle.Information)

            If Not String.IsNullOrEmpty(fullName) Then
                EW.SetPropertyString(ExifWorks.TagNames.Artist, fullName & " (" & uLogin.ToUpper & ")")
            Else
                EW.SetPropertyString(ExifWorks.TagNames.Artist, uLogin.ToUpper)
            End If

            EW.GetBitmap.Save(newFileName)

            theTagList = getTagListFromListBox(Me.clstExifTags)
            'Next line is the OFFICIAL line of code to use so far:
            'SaveTags(theTagList, newFileName)

            Dim [property] As StructuredStorage.SummaryProperty = DirectCast([Enum].Parse(GetType(StructuredStorage.SummaryProperty), DirectCast("Keywords", String)), StructuredStorage.SummaryProperty)
            Using storage As New StructuredStorage.StructuredStorage(newFileName)
                storage.WriteProperty([property], theTagList)
            End Using


            'Now populate the data structure and pass it to the subroutine for
            'insertion into the Access database
            Dim gTagInfo As New geoTaggedInfo

            gTagInfo.keywords = theTagList
            gTagInfo.taggedByUserName = uLogin.ToUpper
            gTagInfo.tageedByFullName = fullName
            gTagInfo.dateTagged = FormatDateTime(Now, DateFormat.ShortDate)
            gTagInfo.description = Me.txtImgDesc.Text
            gTagInfo.longitude = Me.txtLongitude.Text
            gTagInfo.latitude = Me.txtLatitude.Text
            gTagInfo.fullpath = curFileName
            gTagInfo.filename = System.IO.Path.GetFileName(curFileName)
            gTagInfo.taggedByDepartment = olContactInfo.department
            gTagInfo.taggedByDivision = olContactInfo.division
            gTagInfo.taggedByWorkgroup = olContactInfo.workgroup

            addPhotoToGeoDatabase(gTagInfo)
            EW.Dispose()
            EW = Nothing

            Me.imgBox.Image = Nothing
            GC.Collect()
            GC.WaitForPendingFinalizers()

            'Uncomment the next two lines once everything else is working as designed
            My.Computer.FileSystem.DeleteFile(curFileName)
            My.Computer.FileSystem.RenameFile(newFileName, IO.Path.GetFileName(curFileName))
            MsgBox("The EXIF information for this image has been updated.", MsgBoxStyle.Information)

            Dim theFileStream As New System.IO.FileStream(photoPathCol(curImgNum), FileMode.Open, FileAccess.Read)
            Dim myImage As Image = Image.FromStream(theFileStream)

            Me.imgBox.Image = myImage
            myImage = Nothing
            theFileStream.Dispose()
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub cmdActivateLatLong_Click(ByVal sender As Object, ByVal e As System.EventArgs)
        'This code dervied from this ESRI forum posting:
        'http://forums.arcgis.com/threads/31897-Activate-Add-In-Tool?highlight=add+tool+form
        Try
            Dim pUID As New ESRI.ArcGIS.esriSystem.UID
            Dim pCommandItem As ESRI.ArcGIS.Framework.ICommandItem
            Dim m_app As ESRI.ArcGIS.Framework.IApplication

            m_app = My.ArcMap.Application

            pUID.Value = My.ThisAddIn.IDs.getClickLatLong   'Substitute in the name of your tool
            pCommandItem = m_app.Document.CommandBars.Find(pUID, False, False)
            m_app.CurrentTool = pCommandItem
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub frmPhotoDlg_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Try
            If isOutLookInstalled() Then
                Dim fullName As String
                Dim olContactInfo As New oLookContactInfo

                olContactInfo = getOutlookName()
                fullName = olContactInfo.fullName

                If Not String.IsNullOrEmpty(fullName) Then
                    Me.Text = "Photo GeoTagging Tool (Version 1.0) - " & fullName
                Else
                    Me.Text = "Photo GeoTagging Tool (Version 1.0)"
                End If
            End If

            fillCheckListboxFromFile(Me.clstExifTags)

        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub cmdReuseLastCoord_Click(ByVal sender As Object, ByVal e As System.EventArgs)
        Try
            Me.txtLatitude.Text = lastLat.ToString
            Me.txtLongitude.Text = lastLong.ToString
            If Not cmdSaveChanges.Enabled Then Me.cmdSaveChanges.Enabled = True
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub cmdCloseFrm_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdCloseFrm.Click
        Me.Close()
    End Sub

    Private Sub cmdManageTags_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdManageTags.Click
        Dim dlgResult As DialogResult
        Dim editForm As New frmEditExifTags
        Dim imgTagList As String = ""

        dlgResult = editForm.ShowDialog()

        If dlgResult = Windows.Forms.DialogResult.OK Then
            'Changes were made so reset and reload the tag listbox
            imgTagList = getFileTags(Me.imgBox.Image)

            fillCheckListboxFromFile(Me.clstExifTags)
            updateExifTagsFromPhoto(imgTagList, Me.clstExifTags)
        End If
    End Sub

    Private Sub cmdActivateLatLong_Click1(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdActivateLatLong.Click
        Dim pUID As New ESRI.ArcGIS.esriSystem.UID
        Dim pCommandItem As ESRI.ArcGIS.Framework.ICommandItem
        Dim m_application As ESRI.ArcGIS.Framework.IApplication

        m_application = My.ArcMap.Application
        pUID.Value = My.ThisAddIn.IDs.getClickLatLong  'Substitute in the name of your tool
        pCommandItem = m_application.Document.CommandBars.Find(pUID, False, False)
        m_application.CurrentTool = pCommandItem
    End Sub

    Private Sub cmdGetKeywordID_Click(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim curImage As Image = Me.imgBox.Image
        Dim imgProperties As PropertyItem() = curImage.PropertyItems
        'Dim i As Integer
        Dim curFileName As String
        Dim curImgNum As Integer

        curImgNum = Me.imgBox.Tag
        curFileName = photoPathCol(curImgNum)

        Dim EW As New ExifWorks(curFileName)

        Dim fInfo As New FileInfo(curFileName)
        Dim tags As String = ReadFileAttributes(fInfo)
        MsgBox("File Info: " & tags & vbNewLine & "Length: " & Len(tags), MsgBoxStyle.Information)

        'MsgBox("Keywords Value: " + EW.GetPropertyString(40094), MsgBoxStyle.Information)
        'MsgBox("There are " & imgProperties.Length & " image properties.", MsgBoxStyle.Information)
        'For i = 0 To imgProperties.Length - 1
        ' MsgBox("Id :" + imgProperties(i).Id.ToString & vbNewLine & "Value:" + EW.GetPropertyString(imgProperties(i).Id), MsgBoxStyle.Information)
        ' Next i
    End Sub
    Public Function getFileTags(ByVal theImage As Image) As String
        Try
            Dim imgProperties As PropertyItem() = theImage.PropertyItems
            Dim curFileName As String
            Dim curImgNum As Integer

            curImgNum = Me.imgBox.Tag
            curFileName = photoPathCol(curImgNum)

            Dim EW As New ExifWorks(curFileName)

            Dim fInfo As New FileInfo(curFileName)
            Dim tags As String = ReadFileAttributes(fInfo)
            Return tags

            EW = Nothing
        Catch ex As Exception
            globalErrorHandler(ex)
            Return ""
        End Try
    End Function

    Private Sub cmdLastPhoto_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdLastPhoto.Click
        Try
            Dim curImgNum As Integer
            Dim curFileName As String
            curImgNum = Me.imgBox.Tag

            Dim theFileStream As New System.IO.FileStream(photoPathCol(photoPathCol.Count), FileMode.Open, FileAccess.Read)
            Dim myImage As Image = Image.FromStream(theFileStream)

            Me.imgBox.Image = myImage
            myImage = Nothing
            theFileStream.Dispose()

            'Me.imgBox.Image = imgCol(imgCol.Count)
            Me.txtImgName.Text = IO.Path.GetFileNameWithoutExtension(photoPathCol(imgCol.Count))
            Me.imgBox.Tag = imgCol.Count
            Me.lblPhotoCount.Text = "Photo " & Me.imgBox.Tag & " of " & imgCol.Count

            curFileName = photoPathCol(imgCol.Count)

            Dim EW As New ExifWorks(curFileName)
            Dim GPSLat As Double = EW.GPSLatitude()
            Dim GPSLong As Double = EW.GPSLongitude()
            Dim imgTagList As String = ""

            If Not Double.IsNaN(GPSLat) Then
                Me.txtLatitude.Text = GPSLat.ToString
            Else
                Me.txtLatitude.Text = ""
            End If
            If Not Double.IsNaN(GPSLong) Then
                Me.txtLongitude.Text = GPSLong.ToString
            Else
                Me.txtLongitude.Text = ""
            End If

            If lastLat <> 0 Then
                Me.cmdReuseLastCoord.Enabled = True
            Else
                Me.cmdReuseLastCoord.Enabled = False
            End If

            Dim theDescBytes As Byte() = EW.GetProperty(ExifWorks.TagNames.ExifUserComment)

            If theDescBytes Is Nothing Then
                Me.txtImgDesc.Text = String.Empty
            Else
                Me.txtImgDesc.Text = System.Text.Encoding.Unicode.GetString(theDescBytes)
            End If

            'imgTagList = EW.GetPropertyString(40094) 'ReadFileTags(curFileName)
            imgTagList = ReadFileTags(curFileName)

            'Dim theTagBytes As Byte()
            Dim imgMD As New ImageMetadata

            imgMD = ReadEXIFMetadata(curFileName)
            'imgTagList = ReadFileTags(curFileName) 'imgMD.Keywords
            ''imgTagList = ReadTags(curFileName)
            Dim [property] As StructuredStorage.SummaryProperty = DirectCast([Enum].Parse(GetType(StructuredStorage.SummaryProperty), DirectCast("Comments", String)), StructuredStorage.SummaryProperty)
            Using storage As New StructuredStorage.StructuredStorage(curFileName)
                imgTagList = storage.ReadProperty([property])
            End Using

            'imgTagList = ReadFileTags(curFileName)

            'theTagBytes = EW.GetProperty(40094)
            'imgTagList = System.Text.Encoding.Unicode.GetString(theTagBytes)
            MsgBox("Image Tag List: ##" & imgTagList & "##", MsgBoxStyle.Information)

            If Len(imgTagList) > 0 Then
                updateExifTagsFromPhoto(imgTagList, Me.clstExifTags)
                Me.clstExifTags.Tag = imgTagList
            Else
                fillCheckListboxFromFile(Me.clstExifTags)
                Me.clstExifTags.Tag = ""
            End If

            If Me.cmdSaveChanges.Enabled Then Me.cmdSaveChanges.Enabled = False
            EW = Nothing
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub cmdFirstPhoto_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdFirstPhoto.Click
        Try
            Dim curImgNum As Integer
            Dim curFileName As String
            curImgNum = Me.imgBox.Tag

            Dim theFileStream As New System.IO.FileStream(photoPathCol(1), FileMode.Open, FileAccess.Read)
            Dim myImage As Image = Image.FromStream(theFileStream)

            Me.imgBox.Image = myImage
            myImage = Nothing
            theFileStream.Dispose()

            'Me.imgBox.Image = imgCol(1)
            Me.txtImgName.Text = IO.Path.GetFileNameWithoutExtension(photoPathCol(1))
            Me.imgBox.Tag = 1
            Me.lblPhotoCount.Text = "Photo " & Me.imgBox.Tag & " of " & imgCol.Count

            curFileName = photoPathCol(1)

            Dim EW As New ExifWorks(curFileName)
            Dim GPSLat As Double = EW.GPSLatitude()
            Dim GPSLong As Double = EW.GPSLongitude()
            Dim imgTagList As String = ""

            If Not Double.IsNaN(GPSLat) Then
                Me.txtLatitude.Text = GPSLat.ToString
            Else
                Me.txtLatitude.Text = ""
            End If
            If Not Double.IsNaN(GPSLong) Then
                Me.txtLongitude.Text = GPSLong.ToString
            Else
                Me.txtLongitude.Text = ""
            End If

            If lastLat <> 0 Then
                Me.cmdReuseLastCoord.Enabled = True
            Else
                Me.cmdReuseLastCoord.Enabled = False
            End If

            Dim theDescBytes As Byte() = EW.GetProperty(ExifWorks.TagNames.ExifUserComment)

            If theDescBytes Is Nothing Then
                Me.txtImgDesc.Text = String.Empty
            Else
                Me.txtImgDesc.Text = System.Text.Encoding.Unicode.GetString(theDescBytes)
            End If

            imgTagList = EW.GetPropertyString(40094) 'ReadFileTags(curFileName)

            If Len(imgTagList) > 0 Then
                updateExifTagsFromPhoto(imgTagList, Me.clstExifTags)
                Me.clstExifTags.Tag = imgTagList
            Else
                fillCheckListboxFromFile(Me.clstExifTags)
                Me.clstExifTags.Tag = ""
            End If

            If Me.cmdSaveChanges.Enabled Then Me.cmdSaveChanges.Enabled = False
            EW = Nothing
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub txtLatitude_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtLatitude.Validated
        Try
            lastLat = Me.txtLatitude.Text
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub txtLongitude_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtLongitude.Validated
        Try
            lastLong = Me.txtLongitude.Text
        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub Button1_Click(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim oSummProps As DSOFile.SummaryProperties
        Dim m_oDocument As DSOFile.OleDocumentPropertiesClass
        Dim curFileName As String
        Dim curImgNum As Integer

        Try

            curImgNum = Me.imgBox.Tag
            curFileName = photoPathCol(curImgNum)

            m_oDocument = New DSOFile.OleDocumentPropertiesClass
            m_oDocument.Open(curFileName, True, DSOFile.dsoFileOpenOptions.dsoOptionOpenReadOnlyIfNoWriteAccess)

            oSummProps = m_oDocument.SummaryProperties

            MsgBox("Keywords: " & oSummProps.Keywords, MsgBoxStyle.Information)

        Catch ex As Exception
            globalErrorHandler(ex)
        End Try
    End Sub

    Private Sub clstExifTags_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles clstExifTags.Validated
        If Not Me.cmdSaveChanges.Enabled Then Me.cmdSaveChanges.Enabled = True
    End Sub
End Class